package api

import (
	"bytes"
	"compress/gzip"
	"encoding/json"
	"fmt"
	"io"
	"mime/multipart"
	"net/http"
	"strings"

	"github.com/gin-gonic/gin"
)

const (
	CASDOOEN_HOST = "https://auth.3dqueen.cloud"
)

// forwardRequestWithJWT forwards the request to target URL with JWT token and original request data
func forwardRequestWithJWT(c *gin.Context, targetURL string) (interface{}, error) {
	jwtToken := c.GetHeader("Authorization")
	fmt.Println(c.Copy().GetHeader("Content-Type"))

	// Check if the request is multipart
	if c.Request.MultipartForm != nil || strings.HasPrefix(c.GetHeader("Content-Type"), "multipart/form-data") {
		err := c.Request.ParseMultipartForm(32 << 20) // 32MB max memory
		if err != nil {
			return nil, fmt.Errorf("parse multipart form error: %v", err)
		}

		// Create a pipe to write the form data
		pr, pw := io.Pipe()
		writer := multipart.NewWriter(pw)

		// Start goroutine to write form data
		go func() {
			defer pw.Close()
			defer writer.Close()

			// Copy all form fields
			for key, values := range c.Request.MultipartForm.Value {
				for _, value := range values {
					writer.WriteField(key, value)
				}
			}

			// Copy all files
			for key, fileHeaders := range c.Request.MultipartForm.File {
				for _, fileHeader := range fileHeaders {
					part, err := writer.CreateFormFile(key, fileHeader.Filename)
					if err != nil {
						pw.CloseWithError(err)
						return
					}
					file, err := fileHeader.Open()
					if err != nil {
						pw.CloseWithError(err)
						return
					}
					if _, err := io.Copy(part, file); err != nil {
						file.Close()
						pw.CloseWithError(err)
						return
					}
					file.Close()
				}
			}
		}()

		// Create new request with the piped body
		req, err := http.NewRequest(c.Request.Method, targetURL, pr)
		if err != nil {
			return nil, fmt.Errorf("create request error: %v", err)
		}

		// Set headers
		req.Header.Set("Content-Type", writer.FormDataContentType())
		req.Header.Set("Authorization", jwtToken)

		// Make the request
		client := &http.Client{}
		resp, err := client.Do(req)
		if err != nil {
			return nil, fmt.Errorf("do request error: %v", err)
		}
		defer resp.Body.Close()

		// Read response body
		body, err := io.ReadAll(resp.Body)
		if err != nil {
			return nil, fmt.Errorf("read response body error: %v", err)
		}

		// Parse response
		var result interface{}
		if err := json.Unmarshal(body, &result); err != nil {
			return nil, fmt.Errorf("parse response error: %v", err)
		}

		return result, nil
	}

	// Handle non-multipart requests
	var req *http.Request
	var err error

	// Read and reuse the original body if it's a POST request
	var bodyBytes []byte
	if c.Request.Body != nil {
		bodyBytes, err = io.ReadAll(c.Request.Body)
		if err != nil {
			return nil, fmt.Errorf("read request body error: %v", err)
		}
		// Restore the body for later use
		c.Request.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))
	}

	// Create new request with original method and body
	req, err = http.NewRequest(c.Request.Method, targetURL, bytes.NewBuffer(bodyBytes))
	if err != nil {
		return nil, fmt.Errorf("create request error: %v", err)
	}

	req.Header.Set("Authorization", jwtToken)

	if c.Request.Method == "POST" {
		req.Header.Set("Content-Type", "application/json")
	}

	// Accept gzip encoding
	req.Header.Set("Accept-Encoding", "gzip")

	// Send request
	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		return nil, fmt.Errorf("send request error: %v", err)
	}
	defer resp.Body.Close()

	// Read response body, handling gzip if necessary
	var reader io.ReadCloser
	switch resp.Header.Get("Content-Encoding") {
	case "gzip":
		reader, err = gzip.NewReader(resp.Body)
		if err != nil {
			return nil, fmt.Errorf("create gzip reader error: %v", err)
		}
		defer reader.Close()
	default:
		reader = resp.Body
	}

	// Read response
	respBody, err := io.ReadAll(reader)
	if err != nil {
		return nil, fmt.Errorf("read response error: %v", err)
	}

	// Parse response JSON
	var result interface{}
	if err := json.Unmarshal(respBody, &result); err != nil {
		// If JSON parsing fails, return the raw response as string
		return string(respBody), nil
	}
	return result, nil
}

// 转发请求到https://auth.3dqueen.cloud/api/get-account
// 添加jwt认证头
func GetAccount(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return forwardRequestWithJWT(c, CASDOOEN_HOST+"/api/get-account")
}

// 转发请求到https://auth.3dqueen.cloud/api/logout
// 添加jwt认证头
func Logout(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return forwardRequestWithJWT(c, CASDOOEN_HOST+"/api/logout")
}

// GetUsers forwards request to https://auth.3dqueen.cloud/api/get-users
func GetUsers(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	url := fmt.Sprintf(CASDOOEN_HOST+"/api/get-users?%s", c.Request.URL.RawQuery)
	return forwardRequestWithJWT(c, url)
}

// AddUser forwards request to https://auth.3dqueen.cloud/api/add-user
func AddUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return forwardRequestWithJWT(c, CASDOOEN_HOST+"/api/add-user")
}

// UpadteUser forwards request to https://auth.3dqueen.cloud/api/update-user
func UpadteUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	url := fmt.Sprintf(CASDOOEN_HOST+"/api/update-user?%s", c.Request.URL.RawQuery)
	return forwardRequestWithJWT(c, url)
}

// DeleteUser forwards request to https://auth.3dqueen.cloud/api/delete-user
func DeleteUser(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return forwardRequestWithJWT(c, CASDOOEN_HOST+"/api/delete-user")
}

// SetPassword forwards request to https://auth.3dqueen.cloud/api/set-password
func SetPassword(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return forwardRequestWithJWT(c, CASDOOEN_HOST+"/api/set-password")
}
